@@ -28,12 +28,16 @@ class IntermediateValue(object):
     of output processors. "Intermediate" is shorten as "imd" in later naming.
 
     '''
-    def __init__(self, local_name, value, text, context, node=None):
+    def __init__(self, local_name, value, text, context, node=None, start_date=None,
+                 end_date=None, instant=None):
         self.local_name = local_name
         self.value = value
         self.text = text
         self.context = context
         self.node = node
+        self.start_date = start_date
+        self.end_date = end_date
+        self.instant = instant
 
     def __cmp__(self, other):
         if self.value < other.value:
@@ -62,8 +66,9 @@ class ExtractText(object):
 
 class MatchEndDate(object):
 
-    def __init__(self, data_type=str):
+    def __init__(self, data_type=str, ignore_date_range=False):
         self.data_type = data_type
+        self.ignore_date_range = ignore_date_range
 
     def __call__(self, value, loader_context):
         if not hasattr(value, 'select'):
@@ -84,27 +89,28 @@ class MatchEndDate(object):
             log.msg(u'Cannot find context: %s in %s' % (context_id, url), log.WARNING)
             return None
 
-        date = None
+        date = instant = start_date = end_date = None
         try:
-            date = context.xpath('.//*[local-name()="instant"]/text()')[0].extract().strip()
+            instant = context.xpath('.//*[local-name()="instant"]/text()')[0].extract().strip()
         except (IndexError, ValueError):
             try:
-                start_date_str = context.xpath('.//*[local-name()="startDate"]/text()')[0].extract().strip()
                 end_date_str = context.xpath('.//*[local-name()="endDate"]/text()')[0].extract().strip()
-                start_date = datetime.strptime(start_date_str, DATE_FORMAT)
                 end_date = datetime.strptime(end_date_str, DATE_FORMAT)
-                delta_days = (end_date - start_date).days
-                if doc_type == '10-Q' and delta_days < 120 and delta_days > 60:
-                    date = end_date
-                elif doc_type == '10-K' and delta_days < 380 and delta_days > 350:
+
+                start_date_str = context.xpath('.//*[local-name()="startDate"]/text()')[0].extract().strip()
+                start_date = datetime.strptime(start_date_str, DATE_FORMAT)
+
+                if self.ignore_date_range or date_range_matches_doc_type(doc_type, start_date, end_date):
                     date = end_date
             except (IndexError, ValueError):
                 pass
         else:
             try:
-                date = datetime.strptime(date, DATE_FORMAT)
+                instant = datetime.strptime(instant, DATE_FORMAT)
             except ValueError:
                 pass
+            else:
+                date = instant
 
         if date:
             doc_end_date = datetime.strptime(doc_end_date_str, DATE_FORMAT)
@@ -117,7 +123,8 @@ class MatchEndDate(object):
                     pass
                 else:
                     local_name = value.xpath('local-name()')[0].extract()
-                    return IntermediateValue(local_name, val, text, context, value)
+                    return IntermediateValue(local_name, val, text, context, value,
+                        start_date=start_date, end_date=end_date, instant=instant)
 
         return None
 
@@ -144,6 +151,12 @@ class ImdSumMembersOr(object):
         return None
 
 
+def date_range_matches_doc_type(doc_type, start_date, end_date):
+    delta_days = (end_date - start_date).days
+    return ((doc_type == '10-Q' and delta_days < 120 and delta_days > 60) or
+            (doc_type == '10-K' and delta_days < 380 and delta_days > 350))
+
+
 def get_amend(values):
     if values:
         return values[0]
@@ -182,6 +195,29 @@ def imd_get_net_income(imd_values):
     return imd_min(imd_values)
 
 
+def imd_get_op_income(imd_values):
+    imd_values = filter(lambda v: memberness(v.context) < 2, imd_values)
+    return imd_min(imd_values)
+
+
+def imd_get_cash_flow(imd_values, loader_context):
+    if len(imd_values) == 1:
+        return imd_values[0].value
+
+    doc_type = loader_context['doc_type']
+
+    within_date_range = []
+    for imd_value in imd_values:
+        if imd_value.start_date and imd_value.end_date:
+            if date_range_matches_doc_type(doc_type, imd_value.start_date, imd_value.end_date):
+                within_date_range.append(imd_value)
+
+    if within_date_range:
+        return imd_max(within_date_range)
+
+    return imd_max(imd_values)
+
+
 def imd_get_per_share_value(imd_values):
     if not imd_values:
         return None
@@ -232,11 +268,6 @@ def imd_filter_member(imd_values):
 
     return imd_values
 
-    non_members = filter(lambda v: not v.is_member(), imd_values)
-    if non_members:
-        return non_members
-    return imd_values
-
 
 def imd_mult(imd_values):
     for v in imd_values:
@@ -261,7 +292,11 @@ def memberness(context):
         texts = context.xpath('.//*[local-name()="explicitMember"]/text()').extract()
         text = str(texts).lower()
 
-        if 'member' not in text:
+        if len(texts) > 1:
+            return 2
+        elif 'country' in text:
+            return 2
+        elif 'member' not in text:
             return 0
         elif 'successor' in text:
             # 'SuccessorMember' is a rare case that shouldn't be treated as member
@@ -354,6 +389,9 @@ class ReportItemLoader(XmlXPathItemLoader):
     net_income_in = MapCompose(MatchEndDate(float))
     net_income_out = Compose(imd_filter_member, imd_mult, imd_get_net_income)
 
+    op_income_in = MapCompose(MatchEndDate(float))
+    op_income_out = Compose(imd_filter_member, imd_mult, imd_get_op_income)
+
     eps_basic_in = MapCompose(MatchEndDate(float))
     eps_basic_out = Compose(ImdSumMembersOr(imd_get_per_share_value), lambda x: x if x < MAX_PER_SHARE_VALUE else None)
 
@@ -366,12 +404,27 @@ class ReportItemLoader(XmlXPathItemLoader):
     assets_in = MapCompose(MatchEndDate(float))
     assets_out = Compose(imd_filter_member, imd_mult, imd_max)
 
+    cur_assets_in = MapCompose(MatchEndDate(float))
+    cur_assets_out = Compose(imd_filter_member, imd_mult, imd_max)
+
+    cur_liab_in = MapCompose(MatchEndDate(float))
+    cur_liab_out = Compose(imd_filter_member, imd_mult, imd_max)
+
     equity_in = MapCompose(MatchEndDate(float))
     equity_out = Compose(imd_filter_member, imd_mult, imd_get_equity)
 
     cash_in = MapCompose(MatchEndDate(float))
     cash_out = Compose(imd_filter_member, imd_mult, imd_max)
 
+    cash_flow_op_in = MapCompose(MatchEndDate(float, True))
+    cash_flow_op_out = Compose(imd_filter_member, imd_mult, imd_get_cash_flow)
+
+    cash_flow_inv_in = MapCompose(MatchEndDate(float, True))
+    cash_flow_inv_out = Compose(imd_filter_member, imd_mult, imd_get_cash_flow)
+
+    cash_flow_fin_in = MapCompose(MatchEndDate(float, True))
+    cash_flow_fin_out = Compose(imd_filter_member, imd_mult, imd_get_cash_flow)
+
     def __init__(self, *args, **kwargs):
         response = kwargs.get('response')
         if len(response.body) > THRESHOLD_TO_CLEAN:
@@ -443,6 +496,10 @@ class ReportItemLoader(XmlXPathItemLoader):
             '//*[starts-with(local-name(), "NetIncomeAttributableTo")]'
         ])
 
+        self.add_xpaths('op_income', [
+            '//us-gaap:OperatingIncomeLoss'
+        ])
+
         self.add_xpaths('eps_basic', [
             '//us-gaap:EarningsPerShareBasic',
             '//us-gaap:IncomeLossFromContinuingOperationsPerBasicShare',
@@ -492,6 +549,14 @@ class ReportItemLoader(XmlXPathItemLoader):
             '//us-gaap:LiabilitiesAndStockholdersEquity'
         ])
 
+        self.add_xpaths('cur_assets', [
+            '//us-gaap:AssetsCurrent'
+        ])
+
+        self.add_xpaths('cur_liab', [
+            '//us-gaap:LiabilitiesCurrent'
+        ])
+
         self.add_xpaths('equity', [
             '//*[local-name()="StockholdersEquityIncludingPortionAttributableToNoncontrollingInterest" or local-name()="StockholdersEquity"]',
             '//*[local-name()="TotalCommonShareholdersEquity"]',
@@ -515,6 +580,21 @@ class ReportItemLoader(XmlXPathItemLoader):
             '//*[contains(local-name(), "CashAndCashEquivalents")]'
         ])
 
+        self.add_xpaths('cash_flow_op', [
+            '//us-gaap:NetCashProvidedByUsedInOperatingActivities',
+            '//us-gaap:NetCashProvidedByUsedInOperatingActivitiesContinuingOperations'
+        ])
+
+        self.add_xpaths('cash_flow_inv', [
+            '//us-gaap:NetCashProvidedByUsedInInvestingActivities',
+            '//us-gaap:NetCashProvidedByUsedInInvestingActivitiesContinuingOperations'
+        ])
+
+        self.add_xpaths('cash_flow_fin', [
+            '//us-gaap:NetCashProvidedByUsedInFinancingActivities',
+            '//us-gaap:NetCashProvidedByUsedInFinancingActivitiesContinuingOperations'
+        ])
+
     def _get_symbol(self):
         try:
             filename = self.context['response'].url.split('/')[-1]
